package com.mt.wms.basic.controller;

import com.mt.wms.basic.params.AlarmBaseParam;
import com.mt.wms.basic.params.AlarmBaseQueryParam;
import com.mt.wms.basic.params.CraftInfoParam;
import com.mt.wms.basic.params.CraftInfoQueryParam;
import com.mt.wms.basic.service.AlarmBaseService;
import com.mt.wms.basic.service.CraftInfoService;
import com.mt.wms.basic.vo.AlarmBaseVo;
import com.mt.wms.basic.vo.CraftInfoVo;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.validator.groups.AddGroup;
import com.mt.wms.core.validator.groups.PageGroup;
import com.mt.wms.core.validator.groups.UpdateGroup;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.groups.Default;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/10/9 11:09
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "craftInfo")
@Slf4j
@Api(value = "工艺基础信息管理", tags = "工艺基础信息管理", hidden = false)
public class CraftInfoController extends BaseController {

    @Autowired
    private CraftInfoService craftInfoService;

    @PostMapping(value = "get")
    @ApiOperation(value = "获取报警基础信息")
    private R<CraftInfoVo> get(@Validated @RequestBody IdParam idParam) {
        return craftInfoService.get(idParam);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取报警基础信息列表")
    private R<List<CraftInfoVo>> list(@Validated({Default.class}) @RequestBody CraftInfoQueryParam craftInfoQueryParam) {
        return craftInfoService.list(craftInfoQueryParam);
    }

    @PostMapping(value = "page")
    @ApiOperation(value = "获取分页报警基础信息")
    private R<PageVo<CraftInfoVo>> page(@Validated({PageGroup.class, Default.class}) @RequestBody CraftInfoQueryParam craftInfoQueryParam) {
        return craftInfoService.page(craftInfoQueryParam);
    }

    @PostMapping(value = "add")
    @ApiOperation(value = "新增")
    private R<IdVo> add(@Validated({AddGroup.class, Default.class}) @RequestBody CraftInfoParam craftInfoParam) {
        return craftInfoService.add(craftInfoParam);
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    private R<IdVo> update(@Validated({UpdateGroup.class, Default.class}) @RequestBody CraftInfoParam craftInfoParam) {
        return craftInfoService.update(craftInfoParam);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除报警基础信息")
    private R<IdVo> delete(@Validated @RequestBody IdParam idParam) {
        return craftInfoService.delete(idParam);
    }

    @PostMapping(value = "codeGenerator")
    @ApiOperation(value = "编码生成")
    private R<String> codeGenerator() {
        return successful(IDGenerator.gen("GY", "yyyyMMddHHmm", 2, "CRAFTINFO_CODE"));
    }
}
