package com.mt.wms.core.dal.entity;

import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.Version;
import com.baomidou.mybatisplus.annotation.TableId;
import java.time.LocalDateTime;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableField;
import java.io.Serializable;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * <p>
 * 菜单表
 * </p>
 *
 * @author mt
 * @since 2021-11-18
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("t_sys_menu")
public class SysMenu extends Model<SysMenu> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键，自增
     */
      @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 父菜单ID
     */
    @TableField("parent_id")
    private Long parentId;

    /**
     * 菜单编码
     */
    @TableField("code")
    private String code;

    /**
     * 菜单名称
     */
    @TableField("name")
    private String name;

    /**
     * 菜单图标
     */
    @TableField("icon")
    private String icon;

    /**
     * 链接地址，可以是页面地址，也可以是函数事件
     */
    @TableField("href")
    private String href;

    /**
     * 请求地址，可以用于前端ajax调用，后端用于做权限控制
     */
    @TableField("url")
    private String url;

    /**
     * 请求类型：GET, HEAD, POST, PUT, PATCH, DELETE, OPTIONS, TRACE
     */
    @TableField("method")
    private String method;

    /**
     * 权限，可以有多个，用逗号分隔，可用于第三方权限框架扩展
     */
    @TableField("permission")
    private String permission;

    /**
     * 分类:1、pc，2、app，3、wechat，4、miniapp
     */
    @TableField("category")
    private Integer category;

    /**
     * 类型:1、module，2、menu，3、button
     */
    @TableField("type")
    private Integer type;

    /**
     * 层级，从1开始
     */
    @TableField("level")
    private Integer level;

    /**
     * 显示顺序
     */
    @TableField("order_num")
    private Integer orderNum;

    /**
     * 启用状态:0 、停用，1、启用
     */
    @TableField("enabled")
    private Integer enabled;

    /**
     * 备注
     */
    @TableField("remark")
    private String remark;

    /**
     * 删除标志，是否有效:1 可用 0不可用
     */
    @TableField("valid")
    @TableLogic
    private Integer valid;

    /**
     * 创建人
     */
    @TableField("creator")
    private Long creator;

    /**
     * 创建人姓名
     */
    @TableField("creator_name")
    private String creatorName;

    /**
     * 创建时间
     */
    @TableField("create_time")
    private LocalDateTime createTime;

    /**
     * 更新人
     */
    @TableField("updater")
    private Long updater;

    /**
     * 更新人姓名
     */
    @TableField("updater_name")
    private String updaterName;

    /**
     * 更新时间
     */
    @TableField("update_time")
    private LocalDateTime updateTime;

    /**
     * 版本号
     */
    @TableField("version")
    @Version
    private Integer version;


    public static final String ID = "id";

    public static final String PARENT_ID = "parent_id";

    public static final String CODE = "code";

    public static final String NAME = "name";

    public static final String ICON = "icon";

    public static final String HREF = "href";

    public static final String URL = "url";

    public static final String METHOD = "method";

    public static final String PERMISSION = "permission";

    public static final String CATEGORY = "category";

    public static final String TYPE = "type";

    public static final String LEVEL = "level";

    public static final String ORDER_NUM = "order_num";

    public static final String ENABLED = "enabled";

    public static final String REMARK = "remark";

    public static final String VALID = "valid";

    public static final String CREATOR = "creator";

    public static final String CREATOR_NAME = "creator_name";

    public static final String CREATE_TIME = "create_time";

    public static final String UPDATER = "updater";

    public static final String UPDATER_NAME = "updater_name";

    public static final String UPDATE_TIME = "update_time";

    public static final String VERSION = "version";

    @Override
    protected Serializable pkVal() {
        return this.id;
    }

}
