/*
 * Copyright (c) 2018.
 * http://www.ulabcare.com
 */

package com.mt.wms.core.enums;

import com.mt.wms.core.base.IEnum;

/**
 * 子系统:0、common，1、system，2、patient
 * 用于限定维护菜单资源关系，公共资源允许所有菜单关联，其他资源只允许对应的菜单关联，如：平台菜单只允许关联到公共资源和平台资源
 *
 * @author jiff
 * @date 2018/11/22
 * @since 1.0
 */
public enum SubsystemEnum implements IEnum<Integer> {
    /**
     * 公用
     */
    COMMON(0, "公用"),
    /**
     * 平台
     */
    PLATFORM(1, "平台");


    private final Integer value;
    private final String label;
    private final String remark;

    SubsystemEnum(final Integer value, final String label) {
        this(value, label, null);
    }

    SubsystemEnum(final Integer value, final String label, final String remark) {
        this.value = value;
        this.label = label;
        this.remark = remark;
    }

    /**
     * @return 数据值
     */
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * @return 标签名
     */
    @Override
    public String getLabel() {
        return label;
    }

    /**
     * @return 备注
     */
    @Override
    public String getRemark() {
        return remark;
    }
}
