package com.mt.wms.empty.service;

import com.mt.wms.core.dal.entity.CurrTask;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.AutoCurrTaskParam;
import com.mt.wms.empty.params.CurrTaskQueryParam;
import com.mt.wms.empty.params.TaskCreateParam;
import com.mt.wms.empty.vo.CurrTaskMainQueryVo;
import com.mt.wms.empty.vo.CurrTaskQueryVo;
import com.mt.wms.empty.vo.CurrTaskVo;

import java.io.IOException;

/**
 * 拆分任务相关服务
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
public interface CurrTaskService {

    /**
     * 内部使用，获取详情任务的关联任务
     *
     * @param idParam 任务id
     * @return 任务实体
     */
    CurrTaskVo getCurrTask(IdParam idParam);

    /**
     * 内部使用，更新任务单号
     *
     * @param id 任务id
     */
    void updateCurrTaskById(CurrTask id);

    /**
     * 内部使用，获取任务实体
     *
     * @param id 任务id
     * @return 任务实体
     */
    CurrTask getCurrTaskById(Long id);

    /**
     * 创建加工任务的主任务信息
     *
     * @param param 主任务信息
     * @return 结果
     */
    R<IdVo> createProcessTask(TaskCreateParam param);

    /**
     * 查询分页信息
     *
     * @param param 查询参数
     * @return 结果
     */
    R<PageVo<CurrTaskQueryVo>> currentTaskPage(CurrTaskQueryParam param);

    /**
     * 完成执行任务 不会物理删除，会将删除值置为0，同时在历史表复制一份记录 方便在订单未完成时查询任务详情
     *
     * @param param id
     * @return 结果
     */
    R<String> completeTask(IdParam param);

    /**
     * 通过生产单编号删除生产单
     *
     * @param sheetNo 生产单号
     * @return 结果
     */
    R<String> deleteBySheetNo(String sheetNo);
    /**
     * 通过id删除生产单
     *
     * @param
     * @return 结果
     */
    R<String> deleteById(IdParam idParam);

    /**
     * 首页 获取当前执行的任务
     *
     * @return 结果
     */
    R<PageVo<CurrTaskMainQueryVo>> currentTaskMainPage(BasePageParam param);

    /**
     * 执行任务
     */
    R<String> runTask(Long taskId) throws InterruptedException, IOException;

    R<String> createAutoTask(AutoCurrTaskParam autoCurrTaskParam);

    //置顶
    R<String> top(IdParam idParam);
}
