package com.mt.wms.empty.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.utils.CodeGeneratorHelper;
import com.mt.wms.core.utils.HttpClient;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.controller.ApmsController;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.vo.ApmsCreateProcessSheetVo;
import com.mt.wms.empty.vo.ApmsEndProcessVo;
import com.mt.wms.empty.vo.ApmsFinishProcessSheetVo;
import com.mt.wms.empty.websocket.WebSocketServer;
import io.swagger.models.auth.In;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @Author: liguanghao
 * @Date: 2021/11/18 20:27
 * @Version 1.0
 */
@Service
@Transactional
public class AsynRunTaskService extends BaseService {
    private final static Logger logger = LoggerFactory.getLogger(AsynRunTaskService.class);

    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;
    @Resource
    private InStockInfoHisServiceBiz inStockInfoHisServiceBiz;
    @Resource
    private TaskDistanceUtils taskDistanceUtils;
    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Resource
    private PointInfoServiceBiz pointInfoServiceBiz;
    @Resource
    private KilnInfoServiceBiz kilnInfoServiceBiz;
    @Resource
    private PlcNameSpaceServiceBiz plcNameSpaceServiceBiz;
    @Resource
    private ParTemValueServiceBiz parTemValueServiceBiz;
    @Resource
    private ParEleValueServiceBiz parEleValueServiceBiz;
    @Resource
    private ParRotSpeedValueServiceBiz parRotSpeedValueServiceBiz;
    @Resource
    private ParGasValueServiceBiz parGasValueServiceBiz;
    @Resource
    private WebSocketServer webSocketServer;
    @Resource
    private ApmsController apmsController;
    @Resource
    private AutoExeTaskServiceBiz autoExeTaskServiceBiz;
    @Resource
    private CurrTaskDetServiceBiz currTaskDetServiceBiz;
    @Resource
    private TaskHisServiceBiz taskHisServiceBiz;

    //工业炉可用未满的情况下，调用车辆起点为提升台终点为工业炉
    @Async("asyncServiceExecutor")
    public void asynRunTask(Long currTaskId, Long vehicleId) throws IOException, InterruptedException {
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask));
        //检查车辆是否冲突，冲突则避让
        rgvStatus(vehicleId, currTask,currTask.getStartPosition(),currTask.getTargetPosition());
        //新建一条执行任务的关系表存放任务执行信息
        RunTask runTask = new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(currTask.getTargetPosition());
        runTask.setBeginTime(LocalDateTime.now());
        runTaskServiceBiz.save(runTask);
        //调用车辆填入起终点,得到车辆执行结果
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        PointInfo pointInfo = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition()));
        Long endPoint = pointInfo.getCode();
        logger.info("加工炉: "+pointInfo.getName()+"可用未满,调用车辆: "+vehicleId+" 起点为提升台终点为工业炉");
        Integer status = sendTaskToRgv(vehicleId, currTaskId, startPoint, endPoint, currTask.getPlcValue());
        logger.info(String.valueOf(status));
        //执行成功,托盘进炉
        if (status == 1) {
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行成功===");
            //更新调度表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setEndTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表状态
            currTask.setIsIn(1);
            currTask.setStatus(1);
            currTask.setRunTaskId(runTask.getId());
            currTask.setUpdateTime(LocalDateTime.now());
            currTask.setBeginTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //程序休眠10秒.防止进炉后允许入炉信号还没消失,就又发了下一个入炉任务.
            Thread.sleep(10000);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            //任务为自动任务第一步或任务为非自动任务时创建生产单，其他情况下开始生产单
            if (isFirstTask(currTask)) {
                //通知apms创建一个生产单
                apmsCreateProcess(currTask);
            } else {
                int secondSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                if (secondSetp>0){
                    //当前任务为自动任务里的第二步，先上报第一步的apms-endProcess
                    AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                    CurrTask firstCurrTask = currTaskServiceBiz.getById(autoExeTask.getFTaskId());
                    firstCurrTask.setIsIn(2);
                    firstCurrTask.setStatus(2);
                    currTaskServiceBiz.updateById(firstCurrTask);
                    apmsEndProcess(firstCurrTask);
                    //再上报第二步开始处理
                    apmsStartProcess(currTask);
                }
                int threeSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.TH_TASK_ID, currTaskId));
                if (threeSetp>0){
                    //当前任务为自动任务里的第三步，先上报第二步的apms处理endProcess
                    AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.TH_TASK_ID, currTaskId));
                    CurrTask secondCurrTask = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                    secondCurrTask.setIsIn(2);
                    secondCurrTask.setStatus(2);
                    currTaskServiceBiz.updateById(secondCurrTask);
                    apmsEndProcess(secondCurrTask);
                    //再上报第三步开始处理
                    apmsStartProcess(currTask);
                }
            }
            asynStartRecordConsume(currTaskId, currTask.getKilnId());
        } else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            currTask.setStatus(0);
            currTaskServiceBiz.updateById(currTask);
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            logger.info(String.valueOf(status));
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            logger.info("任务 " + currTask.getTaskCode() + " 车辆从起点"+startPoint+"台到加工炉过程中失败。");
            webSocketServer.sendtoAll("任务 " + currTask.getTaskCode() + " 车辆从起点"+startPoint+"台到加工炉过程中失败。");
        }
    }

    //工业炉可用未满的情况下，调用车辆起点为缓存区终点为工业炉
    public void asynRunTaskForStockToKiln(Long stockInfokId, Long vehicleId) throws IOException {
        InStockInfo inStockInfo = inStockInfoServiceBiz.getById(stockInfokId);
        Location location = locationServiceBiz.getById(inStockInfo.getLocationId());
        Long currTaskId = inStockInfo.getTaskId();
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask));
        //检查车辆是否冲突，冲突则避让
        rgvStatus(vehicleId, currTask,location.getCode(),currTask.getTargetPosition());
        //新建一条执行任务的关系表存放任务执行信息
        RunTask runTask = new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        // 起点为库位
        runTask.setStartPosition(location.getCode());
        runTask.setEndPosition(currTask.getTargetPosition());
        runTask.setBeginTime(LocalDateTime.now());
        runTaskServiceBiz.save(runTask);
        //修改缓存区存储情况表状态
        inStockInfo.setStatus(1);
        inStockInfoServiceBiz.updateById(inStockInfo);
        // 调用车辆填入起终点,得到车辆执行结果
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        Long endPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition())).getCode();
        Integer status = sendTaskToRgv(vehicleId, currTaskId, startPoint, endPoint, currTask.getPlcValue());
        //执行成功,托盘进炉
        if (status == 1) {

            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行成功===");
            //更新调度表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setEndTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTaskServiceBiz.updateById(runTask);
            //更改缓存区存储状态,添加缓存区存储历史情况记录
            inStockInfo.setStatus(2);
            inStockInfoServiceBiz.updateById(inStockInfo);
            InStockInfoHis inStockInfoHis = new InStockInfoHis();
            BeanUtils.copyProperties(inStockInfo, inStockInfoHis);
            inStockInfoHisServiceBiz.save(inStockInfoHis);
            inStockInfoServiceBiz.removeById(inStockInfo);
            //更改库位状态
            location.setStatus(0);
            locationServiceBiz.updateById(location);
            //更新currTask表状态
            currTask.setIsIn(1);
            currTask.setStatus(1);
            currTask.setRunTaskId(runTask.getId());
            currTask.setUpdateTime(LocalDateTime.now());
            currTask.setBeginTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            //任务为自动任务第一步或任务为非自动任务时创建生产单，其他情况下开始生产单
            if (isFirstTask(currTask)) {
                //通知apms创建一个生产单
                apmsCreateProcess(currTask);
            } else {
                int secondSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                if (secondSetp>0){
                    //从缓存区到加工炉的任务,其上一步已经报告了apmsend
                    //当前任务为自动任务里的第二步，先上报第一步的apms-endProcess
                    //AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                    //CurrTask firstCurrTask = currTaskServiceBiz.getById(autoExeTask.getFTaskId());
                    //firstCurrTask.setIsIn(2);
                    //currTaskServiceBiz.updateById(firstCurrTask);
                    //apmsEndProcess(firstCurrTask);
                    //再上报第二步开始处理
                    apmsStartProcess(currTask);
                }
                int threeSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.TH_TASK_ID, currTaskId));
                if (threeSetp>0){
                    //当前任务为自动任务里的第三步，先上报第二步的apms处理endProcess
                    //AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.TH_TASK_ID, currTaskId));
                    //CurrTask secondCurrTask = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                    //secondCurrTask.setIsIn(2);
                    //currTaskServiceBiz.updateById(secondCurrTask);
                    //apmsEndProcess(secondCurrTask);
                    //再上报第三步开始处理
                    apmsStartProcess(currTask);
                }
            }
            // 记录开始加工时的消耗值
            asynStartRecordConsume(currTaskId, currTask.getKilnId());
        } else {
            //修改缓存区存储情况表状态
            inStockInfo.setStatus(0);
            inStockInfoServiceBiz.updateById(inStockInfo);
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);

            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            logger.info(String.valueOf(status));
            logger.info("任务 " + currTask.getTaskCode() + " 车辆从缓存区到加工炉过程中失败。");
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            // TODO: 2021/12/14 websocket推送到前端,小车管理界面添加一个重置小车状态的接口。
            // TODO: 2021/12/14 websocket推送到前端,执行失败后车辆在中途，缓存区存储情况怎么修改？
            webSocketServer.sendtoAll("任务 " + currTask.getTaskCode() + " 车辆从缓存区到加工炉过程中失败。");
        }
    }

    //工业炉已满，调用车辆起点为提升台，终点为缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskToWarehouse(Long currTaskId, Long vehicleId,Location location) throws IOException {
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        //Location location = locationServiceBiz.getById(currTask.getLocationId());
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        //检查车辆是否冲突，冲突则避让
        rgvStatus(vehicleId, currTask,currTask.getStartPosition(),location.getCode());
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask = new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(location.getCode());
        runTaskServiceBiz.save(runTask);

        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo = new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        inStockInfo.setType(1);
        inStockInfo.setCreateTime(LocalDateTime.now());
        inStockInfoServiceBiz.save(inStockInfo);
        // 调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        Long endPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition())).getCode();
        Integer status = sendTaskToRgv(vehicleId, currTaskId, startPoint, endPoint, currTask.getPlcValue());
        if (status == 1) {
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行成功===");
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            currTask.setCacheSatatus(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            //如果任务为多步骤任务的非第一步，置前置任务isIn为2
            if (currTask.getIsAuto()==1){
                int count = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.F_TASK_ID, currTaskId));
                if (count==0){
                    int secondCount = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                    if (secondCount==1){
                        AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                        CurrTask firstCurrTask = currTaskServiceBiz.getById(autoExeTask.getFTaskId());
                        firstCurrTask.setIsIn(2);
                        firstCurrTask.setStatus(2);
                        currTaskServiceBiz.updateById(firstCurrTask);
                    }else {
                        AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.TH_TASK_ID, currTaskId));
                        CurrTask secondCurrTask = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                        secondCurrTask.setIsIn(2);
                        secondCurrTask.setStatus(2);
                        currTaskServiceBiz.updateById(secondCurrTask);
                    }
                }
            }
        } else {
            //修改库位状态
            location.setStatus(0);
            locationServiceBiz.updateById(location);
            //任务失败清除缓存库位信息
            inStockInfoServiceBiz.removeById(inStockInfo);
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            logger.info(String.valueOf(status));
            logger.info("任务 " + currTask.getTaskCode() + " 车辆从液压台到缓存区过程中失败。");
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            webSocketServer.sendtoAll("任务 " + currTask.getTaskCode() + " 车辆从液压台到缓存区过程中失败。");
        }
    }

    //工业炉加工完成，调用车辆入库到缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskForKilnToWarehouse(Long currTaskId, Long vehicleId) throws IOException {
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        Location location = locationServiceBiz.getOne(new QueryWrapper<Location>().eq(Location.CODE, currTask.getOutTargetPosition()));
        //检查车辆是否冲突，冲突则避让
        rgvStatus(vehicleId, currTask,currTask.getStartPosition(),location.getCode());
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask = new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(currTask.getOutStartPosition());
        runTask.setEndPosition(currTask.getOutTargetPosition());
        runTaskServiceBiz.save(runTask);
        //更新 location 表对应库位状态为占用

        location.setStatus(1);
        location.setUpdateTime(LocalDateTime.now());
        locationServiceBiz.updateById(location);
        currTask.setOutRunTaskId(runTask.getId());
        currTaskServiceBiz.updateById(currTask);
        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo = new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        inStockInfo.setCreateTime(LocalDateTime.now());
        inStockInfoServiceBiz.save(inStockInfo);
        // 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        Long endPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition())).getCode();
        Integer status = sendTaskToRgv(vehicleId, currTaskId, startPoint, endPoint, currTask.getPlcValue());
        if (status == 1) {
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行成功===");
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            currTask.setIsOut(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTask.setStatus(2);
            currTask.setIsIn(2);
            currTask.setUpdateTime(LocalDateTime.now());
            logger.info("修改currTask任务状态");
            currTaskServiceBiz.updateById(currTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //更新小车状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            //任务为非自动任务上报apms-end-finish
            if (currTask.getIsAuto()==0) {
                //上报apms完成处理
                logger.info("此任务为单步骤任务，任务结束，上报apms-end-finish");
                apmsEndProcess(currTask);
                apmsFinishProcess(currTask);
            } else {
                int firstSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.F_TASK_ID, currTaskId));
                if (firstSetp>0){
                    //当前任务为自动任务的第一步
                    apmsEndProcess(currTask);
                }
                int secondSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                if (secondSetp>0){
                    AutoExeTask autoExeTask = autoExeTaskServiceBiz.getOne(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.T_TASK_ID, currTaskId));
                    if (autoExeTask.getProcessFlowType()==1){
                        logger.info("此任务为为自动任务里的第二步且一共只有两步，任务结束，上报apms-end-finish");
                        //当前任务为自动任务里的第三步且一共只有两步，先上报apms处理endProcess
                        apmsEndProcess(currTask);
                        //再上报apms-finishProcess
                        apmsFinishProcess(currTask);
                    }else {
                        //当前任务为三步中的第二步
                        apmsEndProcess(currTask);
                    }
                }
                int threeSetp = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.TH_TASK_ID, currTaskId));
                if (threeSetp>0){
                    logger.info("此任务为为自动任务里的第三步，任务结束，上报apms-end-finish");
                    //当前任务为自动任务里的第三步，先上报apms处理endProcess
                    apmsEndProcess(currTask);
                    //再上报apms-finishProcess
                    apmsFinishProcess(currTask);
                }

            }
        } else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 " + currTask.getTaskCode() + " 车辆从工业炉到缓存区过程中失败。");
            logger.info("===执行任务：" + currTaskId + " ，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            //  2021/12/14 websocket推送到前端
            webSocketServer.sendtoAll("任务 " + currTask.getTaskCode() + " 车辆从工业炉到缓存区过程中失败。");
        }
    }
    //出库。起点为加工炉，终点为提升台
    @Async("asyncServiceExecutor")
    public void asynRunOutStockForKilnToYYT(Long currTaskId, String yeyaTaiCode, Long vehicleId) throws IOException {
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(currTask.getKilnId());
        VehicleInfo vehicle = vehicleInfoServiceBiz.getById(vehicleId);
        if (vehicleId == 1) {
            Long anotherVehicleId = 2L;
            Boolean rgv2Online = rgvOnline(anotherVehicleId);
            if (rgv2Online) {
                //获取RGV2当前位置
                Double rgv2CurrPosition = RGVCurrPosition(2);
                //两辆小车是否冲突
                Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, kilnInfo.getCode(), yeyaTaiCode, rgv2CurrPosition);
                if (conflictBoolean) {
                    Integer status = moveRgv(2, 1L, 2, 0);
                    logger.info(status.toString());
                    if (status != 1) {
                        logger.info("===执行出库任务时车辆位置冲突，执行车辆位移任务，车辆" + anotherVehicleId + "移动到点位2，执行失败===");
                        //  2022/3/6  websocket通知页面
                        webSocketServer.sendtoAll(LocalDateTime.now() + "执行出库任务时车辆位置冲突，执行车辆位移任务，车辆" + anotherVehicleId + "移动到点位2，执行失败");
                    }else {

                        logger.info("===执行任务出库任务时车辆位置冲突，执行车辆位移任务，车辆" + anotherVehicleId + "移动到点位2，执行成功===");
                    }
                }
            } else {
                logger.info("RGV2不在线，跳过检测是否避让。");
            }

        } else {
            Boolean rgv1Online = rgvOnline(1L);
            if (rgv1Online) {
                //获取RGV1当前位置
                Double rgv1CurrPosition = RGVCurrPosition(1);
                //两辆小车是否冲突
                Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(2L, kilnInfo.getCode(), yeyaTaiCode, rgv1CurrPosition);
                if (conflictBoolean) {
                    Integer status = moveRgv(1, 1L, 12, 0);
                    logger.info(status.toString());
                    if (status != 1) {
                        logger.info("===执行任务出库任务时车辆位置冲突，执行车辆位移任务，车辆" + 1 + "移动到点位12，执行失败===");
                        //  2022/3/6  websocket通知页面
                        webSocketServer.sendtoAll(LocalDateTime.now() + "执行出库任务时车辆位置冲突，执行车辆位移任务，车辆" + 1 + "移动到点位12，执行失败");
                    }else {
                        logger.info("===执行任务出库任务时车辆位置冲突，执行车辆位移任务，车辆" + 1 + "移动到点位12，执行成功===");
                    }
                }
            } else {
                logger.info("RGV1不在线，跳过检测是否避让。");
            }
        }
        RunTask runTask = new RunTask();
        // setCommonField(runTask);
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(kilnInfo.getCode());
        runTask.setEndPosition(yeyaTaiCode);
        runTaskServiceBiz.save(runTask);
        //  2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（出库任务，任务号为0）
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        Long endPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition())).getCode();
        Integer status = sendTaskToRgv(vehicleId, 0L, startPoint, endPoint, 0);
        logger.info(status.toString());
        if (status == 1) {
            logger.info("===执行任务出库任务，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行成功===");
            //更新调度任务表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新车辆状态
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            //更新currTask表
            currTask.setIsOut(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTask.setStatus(2);
            currTask.setIsIn(2);
            currTask.setUpdateTime(LocalDateTime.now());
            logger.info("修改currTask任务状态");
            currTaskServiceBiz.updateById(currTask);
            //上报apms
            apmsEndProcess(currTask);
            apmsFinishProcess(currTask);
        } else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            vehicle.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicle);
            logger.info("任务 " + runTask.getTaskCode() + " 车辆从加工炉到提升平台过程中失败。");
            logger.info("===执行任务出库任务，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            //  2021/12/14 websocket推送到前端
            webSocketServer.sendtoAll("出库任务车辆从加工炉到提升平台过程中失败。");
        }
    }
    //出库。起点为缓存区库位，终点为提升台
    @Async("asyncServiceExecutor")
    public void asynRunOutStock(Long locationId, String yeyaTaiCode, Long vehicleId) throws IOException {

        VehicleInfo vehicleInfo = vehicleInfoServiceBiz.getById(vehicleId);
        Location location = locationServiceBiz.getById(locationId);
        if (vehicleId == 1) {
            Long anotherVehicleId = 2L;
            Boolean rgv2Online = rgvOnline(anotherVehicleId);
            if (rgv2Online) {
                //获取RGV2当前位置
                Double rgv2CurrPosition = RGVCurrPosition(2);
                //两辆小车是否冲突
                Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, location.getCode(), yeyaTaiCode, rgv2CurrPosition);
                if (conflictBoolean) {
                    Integer status = moveRgv(2, 1L, 2, 0);
                    if (status != 1) {
                        logger.info("===执行出库任务时车辆位置冲突，执行车辆位移任务，车辆" + vehicleId + "移动到点位39，执行失败===");
                        //  2022/3/6  websocket通知页面
                        webSocketServer.sendtoAll(LocalDateTime.now() + "执行出库任务时车辆位置冲突，执行车辆位移任务，车辆" + vehicleId + "移动到点位39，执行失败");
                        return;
                    }
                    logger.info("===执行任务出库任务时车辆位置冲突，执行车辆位移任务，车辆" + vehicleId + "移动到点位39，执行成功===");
                }
            } else {
                logger.info("RGV2不在线，跳过检测是否避让。");
            }

        } else {
            Boolean rgv1Online = rgvOnline(1L);
            if (rgv1Online) {
                //获取RGV1当前位置
                Double rgv1CurrPosition = RGVCurrPosition(1);
                //两辆小车是否冲突
                Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, location.getCode(), yeyaTaiCode, rgv1CurrPosition);
                if (conflictBoolean) {
                    Integer status = moveRgv(1, 1L, 12, 0);
                    if (status != 1) {
                        logger.info("===执行任务出库任务时车辆位置冲突，执行车辆位移任务，车辆" + vehicleId + "移动到点位1，执行失败===");
                        //  2022/3/6  websocket通知页面
                        webSocketServer.sendtoAll(LocalDateTime.now() + "执行出库任务时车辆位置冲突，执行车辆位移任务，车辆" + vehicleId + "移动到点位1，执行失败");
                        return;
                    }
                    logger.info("===执行任务出库任务时车辆位置冲突，执行车辆位移任务，车辆" + vehicleId + "移动到点位1，执行成功===");
                }
            } else {
                logger.info("RGV1不在线，跳过检测是否避让。");
            }
        }
        RunTask runTask = new RunTask();
        // setCommonField(runTask);
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(location.getCode());
        runTask.setEndPosition(yeyaTaiCode);
        runTaskServiceBiz.save(runTask);
        //修改缓存区状态
        InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>()
                .eq("location_id", locationId));
        inStockInfo.setStatus(0);
        inStockInfoServiceBiz.updateById(inStockInfo);

        //  2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（出库任务，任务号为0）
        Long startPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getStartPosition())).getCode();
        Long endPoint = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, runTask.getEndPosition())).getCode();
        Integer status = sendTaskToRgv(vehicleId, 0L, startPoint, endPoint, 0);
        if (status == 1) {
            logger.info("===执行任务出库任务，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行成功===");
            //更新调度任务表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //缓存区存储历史
            InStockInfoHis inStockInfoHis = new InStockInfoHis();
            BeanUtils.copyProperties(inStockInfo, inStockInfoHis);
            inStockInfoHisServiceBiz.save(inStockInfoHis);
            inStockInfoServiceBiz.removeById(inStockInfo.getId());
            //更新库位状态
            location.setStatus(0);
            locationServiceBiz.updateById(location);

            //更新车辆状态
            vehicleInfo.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicleInfo);
        } else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            vehicleInfo.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicleInfo);
            logger.info("任务 " + runTask.getTaskCode() + " 车辆从缓存区到提升平台过程中失败。");
            logger.info("===执行任务出库任务，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            System.out.println("===执行任务出库任务，车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "任务执行失败===");
            //  2021/12/14 websocket推送到前端
            webSocketServer.sendtoAll("出库任务车辆从缓存区到提升平台过程中失败。");
        }
    }

    //允许进炉信号为true，记录开始加工时的能源消耗数据，再写定时任务记录加工过程的消耗数据，再写一个异步方法，加工任务结束的时候记录一下各项消耗的值
    @Async("asyncServiceExecutor")
    public void asynStartRecordConsume(Long currTaskId, Long kilnId) {
        // TODO: 2022/2/17
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
        PlcNameSpace nameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                .eq(PlcNameSpace.EQ_ID, kilnId)
                .eq(PlcNameSpace.EQ_TYPE, 0));
        String nameSpaceNote = nameSpace.getName();
        switch (kilnInfo.getType()) {
            //加工炉
            case 1:
                //实际温度
                Double actualTemp = readPlc(nameSpaceNote, "ActualTemp");
                //设定温度
                Double setUpTemp = readPlc(nameSpaceNote, "SetUpTemp");
                //油槽实际温度
                Double actualOilTemp = readPlc(nameSpaceNote, "ActualOilTemp");
                //油槽设定温度
                Double setUpOilTemp = readPlc(nameSpaceNote, "SetUpOilTemp");
                ParTemValue parTemValue = new ParTemValue();
                parTemValue.setSetTemValue(setUpTemp.floatValue());
                parTemValue.setActTemValue(actualTemp.floatValue());
                parTemValue.setOilTankSetTemValue(setUpOilTemp.floatValue());
                parTemValue.setOilTankActTemValue(actualOilTemp.floatValue());
                parTemValue.setKilnId(kilnId);
                parTemValue.setTaskId(currTaskId);
                parTemValue.setCreateTime(LocalDateTime.now());
                parTemValueServiceBiz.save(parTemValue);
                //电度值
                Double electricalValue = readPlc(nameSpaceNote, "ElectricalValue");
                //A向电压
                Double phaseAVoltage = readPlc(nameSpaceNote, "PhaseAVoltage");
                Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue = new ParEleValue();
                parEleValue.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue.setTotalBat(electricalValue.floatValue());
                parEleValue.setKilnId(kilnId);
                parEleValue.setTaskId(currTaskId);
                parEleValue.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue);
                //一号油搅拌转速
                Double oilStirringSpeed1 = readPlc(nameSpaceNote, "OilStirringSpeed1");
                //二号油搅拌转速
                Double oilStirringSpeed2 = readPlc(nameSpaceNote, "OilStirringSpeed2");
                ParRotSpeedValue parRotSpeedValue = new ParRotSpeedValue();
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed1.floatValue());
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed2.floatValue());
                parRotSpeedValue.setKilnId(kilnId);
                parRotSpeedValue.setTaskId(currTaskId);
                parRotSpeedValue.setCreateTime(LocalDateTime.now());
                parRotSpeedValueServiceBiz.save(parRotSpeedValue);
                //氮气
                Double nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                //甲烷
                Double methanolFlow = readPlc(nameSpaceNote, "MethanolFlow");
                //丙烷
                Double propaneFlow = readPlc(nameSpaceNote, "PropaneFlow");
                //氨气
                Double ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                //实际碳势
                Double actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定碳势
                Double setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                ParGasValue parGasValue = new ParGasValue();
                parGasValue.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue.setMethanolFlow(methanolFlow.floatValue());
                parGasValue.setPropaneFlow(propaneFlow.floatValue());
                parGasValue.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue.setKilnId(kilnId);
                parGasValue.setTaskId(currTaskId);
                parGasValue.setCreateTime(LocalDateTime.now());
                parGasValueServiceBiz.save(parGasValue);
                break;
            //回火炉
            case 2:
                //实际温度
                actualTemp = readPlc(nameSpaceNote, "ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote, "SetUpTemp");
                ParTemValue parTemValue2 = new ParTemValue();
                parTemValue2.setSetTemValue(setUpTemp.floatValue());
                parTemValue2.setActTemValue(actualTemp.floatValue());
                parTemValue2.setKilnId(kilnId);
                parTemValue2.setTaskId(currTaskId);
                parTemValue2.setCreateTime(LocalDateTime.now());
                parTemValueServiceBiz.save(parTemValue2);
                //电度值
                electricalValue = readPlc(nameSpaceNote, "TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote, "AVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "BVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "CVoltage");
                ParEleValue parEleValue2 = new ParEleValue();
                parEleValue2.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue2.setTotalBat(electricalValue.floatValue());
                parEleValue2.setKilnId(kilnId);
                parEleValue2.setTaskId(currTaskId);
                parEleValue2.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue2);
                break;
            //氮化炉
            case 3:
                //实际温度
                actualTemp = readPlc(nameSpaceNote, "ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote, "SetUpTemp");
                //炉压
                Double furnacePressure = readPlc(nameSpaceNote, "FurnacePressure");
                //外一区温度
                Double outerZoneITemperature = readPlc(nameSpaceNote, "OuterZoneITemperature");
                //外二区温度
                Double outerZone2Temperature = readPlc(nameSpaceNote, "OuterZone2Temperature");
                ParTemValue parTemValue3 = new ParTemValue();
                parTemValue3.setSetTemValue(setUpTemp.floatValue());
                parTemValue3.setActTemValue(actualTemp.floatValue());
                parTemValue3.setOuterZone1Temp(outerZoneITemperature.floatValue());
                parTemValue3.setOuterZone2Temp(outerZone2Temperature.floatValue());
                parTemValue3.setFurnacePressure(furnacePressure.floatValue());
                parTemValue3.setKilnId(kilnId);
                parTemValue3.setTaskId(currTaskId);
                parTemValue3.setCreateTime(LocalDateTime.now());
                parTemValueServiceBiz.save(parTemValue3);
                //电度值
                electricalValue = readPlc(nameSpaceNote, "ElectricalValue");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote, "PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue3 = new ParEleValue();
                parEleValue3.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue3.setTotalBat(electricalValue.floatValue());
                parEleValue3.setKilnId(kilnId);
                parEleValue3.setTaskId(currTaskId);
                parEleValue3.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue3);
                //实际氮势
                actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定氮势
                setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                //氢含量
                Double hydrogenContent = readPlc(nameSpaceNote, "HydrogenContent");
                //分解率
                Double decompositionRate = readPlc(nameSpaceNote, "DecompositionRate");
                //氨气量
                ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                //氮气量
                nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                //二氧化碳量
                Double carbonDioxideFlow = readPlc(nameSpaceNote, "CO2Flow");
                ParGasValue parGasValue3 = new ParGasValue();
                parGasValue3.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue3.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue3.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue3.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue3.setHydrogenContent(hydrogenContent.floatValue());
                parGasValue3.setDecompositionRate(decompositionRate.floatValue());
                parGasValue3.setKilnId(kilnId);
                parGasValue3.setTaskId(currTaskId);
                parGasValue3.setCreateTime(LocalDateTime.now());
                parGasValueServiceBiz.save(parGasValue3);
                break;
            //清洗炉
            case 4:
                //电度值
                electricalValue = readPlc(nameSpaceNote, "TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote, "AVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "BVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "CVoltage");
                ParEleValue parEleValue4 = new ParEleValue();
                parEleValue4.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue4.setTotalBat(electricalValue.floatValue());
                parEleValue4.setKilnId(kilnId);
                parEleValue4.setTaskId(currTaskId);
                parEleValue4.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue4);
                break;
            default:
                logger.info("任务：" + currTaskId + "开始在" + kilnInfo.getKilnName() + "加工，能源消耗开始统计。");
        }
    }

    //允许出炉信号为true，记录结束加工时的能源消耗数据
    @Async("asyncServiceExecutor")
    public void asynEndRecordConsume(Long currTaskId, Long kilnId) {
        // TODO: 2022/2/17
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
        PlcNameSpace nameSpace = plcNameSpaceServiceBiz.getOne(new QueryWrapper<PlcNameSpace>()
                .eq(PlcNameSpace.EQ_ID, kilnId)
                .eq(PlcNameSpace.EQ_TYPE, 0));
        String nameSpaceNote = nameSpace.getName();
        switch (kilnInfo.getType()) {
            //加工炉
            case 1:
                //实际温度
                Double actualTemp = readPlc(nameSpaceNote, "ActualTemp");
                //设定温度
                Double setUpTemp = readPlc(nameSpaceNote, "SetUpTemp");
                //油槽实际温度
                Double actualOilTemp = readPlc(nameSpaceNote, "ActualOilTemp");
                //油槽设定温度
                Double setUpOilTemp = readPlc(nameSpaceNote, "SetUpOilTemp");
                ParTemValue parTemValue = new ParTemValue();
                parTemValue.setSetTemValue(setUpTemp.floatValue());
                parTemValue.setActTemValue(actualTemp.floatValue());
                parTemValue.setOilTankSetTemValue(setUpOilTemp.floatValue());
                parTemValue.setOilTankActTemValue(actualOilTemp.floatValue());
                parTemValue.setKilnId(kilnId);
                parTemValue.setTaskId(currTaskId);
                parTemValue.setCreateTime(LocalDateTime.now());
                parTemValueServiceBiz.save(parTemValue);
                //电度值
                Double electricalValue = readPlc(nameSpaceNote, "ElectricalValue");
                //A向电压
                Double phaseAVoltage = readPlc(nameSpaceNote, "PhaseAVoltage");
                Double phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                Double phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue = new ParEleValue();
                parEleValue.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue.setTotalBat(electricalValue.floatValue());
                parEleValue.setKilnId(kilnId);
                parEleValue.setTaskId(currTaskId);
                parEleValue.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue);
                //一号油搅拌转速
                Double oilStirringSpeed1 = readPlc(nameSpaceNote, "OilStirringSpeed1");
                //二号油搅拌转速
                Double oilStirringSpeed2 = readPlc(nameSpaceNote, "OilStirringSpeed2");
                ParRotSpeedValue parRotSpeedValue = new ParRotSpeedValue();
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed1.floatValue());
                parRotSpeedValue.setOilStiSpeedAValue(oilStirringSpeed2.floatValue());
                parRotSpeedValue.setKilnId(kilnId);
                parRotSpeedValue.setTaskId(currTaskId);
                parRotSpeedValue.setCreateTime(LocalDateTime.now());
                parRotSpeedValueServiceBiz.save(parRotSpeedValue);
                //氮气
                Double nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                //甲烷
                Double methanolFlow = readPlc(nameSpaceNote, "MethanolFlow");
                //丙烷
                Double propaneFlow = readPlc(nameSpaceNote, "PropaneFlow");
                //氨气
                Double ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                //实际碳势
                Double actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定碳势
                Double setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                ParGasValue parGasValue = new ParGasValue();
                parGasValue.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue.setMethanolFlow(methanolFlow.floatValue());
                parGasValue.setPropaneFlow(propaneFlow.floatValue());
                parGasValue.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue.setKilnId(kilnId);
                parGasValue.setTaskId(currTaskId);
                parGasValue.setCreateTime(LocalDateTime.now());
                parGasValueServiceBiz.save(parGasValue);
                break;
            //回火炉
            case 2:
                //实际温度
                actualTemp = readPlc(nameSpaceNote, "ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote, "SetUpTemp");
                ParTemValue parTemValue2 = new ParTemValue();
                parTemValue2.setSetTemValue(setUpTemp.floatValue());
                parTemValue2.setActTemValue(actualTemp.floatValue());
                parTemValue2.setKilnId(kilnId);
                parTemValue2.setTaskId(currTaskId);
                parTemValue2.setCreateTime(LocalDateTime.now());
                parTemValueServiceBiz.save(parTemValue2);
                //电度值
                electricalValue = readPlc(nameSpaceNote, "TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote, "AVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "BVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "CVoltage");
                ParEleValue parEleValue2 = new ParEleValue();
                parEleValue2.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue2.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue2.setTotalBat(electricalValue.floatValue());
                parEleValue2.setKilnId(kilnId);
                parEleValue2.setTaskId(currTaskId);
                parEleValue2.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue2);
                break;
            //氮化炉
            case 3:
                //实际温度
                actualTemp = readPlc(nameSpaceNote, "ActualTemp");
                //设定温度
                setUpTemp = readPlc(nameSpaceNote, "SetUpTemp");
                //炉压
                Double furnacePressure = readPlc(nameSpaceNote, "FurnacePressure");
                //外一区温度
                Double outerZoneITemperature = readPlc(nameSpaceNote, "OuterZoneITemperature");
                //外二区温度
                Double outerZone2Temperature = readPlc(nameSpaceNote, "OuterZone2Temperature");
                ParTemValue parTemValue3 = new ParTemValue();
                parTemValue3.setSetTemValue(setUpTemp.floatValue());
                parTemValue3.setActTemValue(actualTemp.floatValue());
                parTemValue3.setOuterZone1Temp(outerZoneITemperature.floatValue());
                parTemValue3.setOuterZone2Temp(outerZone2Temperature.floatValue());
                parTemValue3.setFurnacePressure(furnacePressure.floatValue());
                parTemValue3.setKilnId(kilnId);
                parTemValue3.setTaskId(currTaskId);
                parTemValue3.setCreateTime(LocalDateTime.now());
                parTemValueServiceBiz.save(parTemValue3);
                //电度值
                electricalValue = readPlc(nameSpaceNote, "ElectricalValue");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote, "PhaseAVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "PhaseBVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "PhaseCVoltage");
                ParEleValue parEleValue3 = new ParEleValue();
                parEleValue3.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue3.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue3.setTotalBat(electricalValue.floatValue());
                parEleValue3.setKilnId(kilnId);
                parEleValue3.setTaskId(currTaskId);
                parEleValue3.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue3);
                //实际氮势
                actualCarbon = readPlc(nameSpaceNote, "ActualCarbon");
                //设定氮势
                setupCarbon = readPlc(nameSpaceNote, "SetupCarbon");
                //氢含量
                Double hydrogenContent = readPlc(nameSpaceNote, "HydrogenContent");
                //分解率
                Double decompositionRate = readPlc(nameSpaceNote, "DecompositionRate");
                //氨气量
                ammoniaFlow = readPlc(nameSpaceNote, "AmmoniaFlow");
                //氮气量
                nitrogenFlow = readPlc(nameSpaceNote, "NitrogenFlow");
                //二氧化碳量
                Double carbonDioxideFlow = readPlc(nameSpaceNote, "CO2Flow");
                ParGasValue parGasValue3 = new ParGasValue();
                parGasValue3.setNitFlowValue(nitrogenFlow.floatValue());
                parGasValue3.setAmmoniaFlowValue(ammoniaFlow.floatValue());
                parGasValue3.setActualNitPotValue(actualCarbon.floatValue());
                parGasValue3.setSetNitPotValue(setupCarbon.floatValue());
                parGasValue3.setHydrogenContent(hydrogenContent.floatValue());
                parGasValue3.setDecompositionRate(decompositionRate.floatValue());
                parGasValue3.setKilnId(kilnId);
                parGasValue3.setTaskId(currTaskId);
                parGasValue3.setCreateTime(LocalDateTime.now());
                parGasValueServiceBiz.save(parGasValue3);
                break;
            //清洗炉
            case 4:
                //电度值
                electricalValue = readPlc(nameSpaceNote, "TotalElectricity");
                //A向电压
                phaseAVoltage = readPlc(nameSpaceNote, "AVoltage");
                phaseBVoltage = readPlc(nameSpaceNote, "BVoltage");
                phaseCVoltage = readPlc(nameSpaceNote, "CVoltage");
                ParEleValue parEleValue4 = new ParEleValue();
                parEleValue4.setAVoltagevValue(phaseAVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseBVoltage.floatValue());
                parEleValue4.setAVoltagevValue(phaseCVoltage.floatValue());
                parEleValue4.setTotalBat(electricalValue.floatValue());
                parEleValue4.setKilnId(kilnId);
                parEleValue4.setTaskId(currTaskId);
                parEleValue4.setCreateTime(LocalDateTime.now());
                parEleValueServiceBiz.save(parEleValue4);
                break;
            default:
                logger.info("任务：" + currTaskId + "开始在" + kilnInfo.getKilnName() + "加工，能源消耗开始统计。");
        }
    }

    /**
     * 获取变量值
     *
     * @param nameSpace    nameSpace前缀
     * @param variableName 变量名称
     * @return 变量值
     */
    private Double readPlc(String nameSpace, String variableName) {
        String identifier = nameSpace.concat(variableName);
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        json.put("identifier", identifier);
        String result = null;
        try {
            result = HttpClient.httpPost("http://192.168.6.51:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("能源消耗统计：获取 " + identifier + " 的值失败");
        }
        return Double.valueOf(JSONObject.parseObject(result).get("result").toString());
    }

    /**
     * RGV1的当前位置
     * id 小车表示
     *
     * @return 小车位置值（距离原点的值）
     */
    private Double RGVCurrPosition(Integer id) {
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        if (id == 1) {
            json.put("identifier", "S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV1ToWCS.CurrentPosition");
        }
        if (id == 2) {
            json.put("identifier", "S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV2ToWCS.CurrentPosition");
        }
        String result = null;
        try {
            result = HttpClient.httpPost("http://192.168.6.51:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("获取RGV1的当前位置失败");
        }
        Double value = Double.valueOf(JSONObject.parseObject(result).get("result").toString());
        return value * 0.0001;
    }

    /**
     * 移动RGV
     *
     * @param vehicleId  车辆id
     * @param currTaskId 任务id
     * @param sendRow    终点
     * @return json字符串
     */
    private Integer moveRgv(Integer vehicleId, Long currTaskId, Integer sendRow, Integer processNumber) {
        //若冲突
        // 执行小车移位任务,移位任务可以直接让小车移位到两端
        Map<String, Object> json = new HashMap();
        //type=1为 入
        // taskType 4 单移动

        json.put("taskType", 4);
        json.put("sendRow", sendRow);
        json.put("pickRow", 0);
        json.put("taskNo", currTaskId);
        json.put("ideNumber", currTaskId);
        json.put("processNumber", processNumber);
        String taskJson = JSON.toJSONString(json);
        logger.info("===执行任务：" + currTaskId + " 时车辆位置冲突，即将执行车辆位移任务，车辆" + vehicleId + "移动到点位" + sendRow + "===");
        String result = null;
        try {
            if (vehicleId == 1) {
                result = HttpClient.httpPost("http://192.168.6.51:8009/rgv1/sendTask", taskJson);
            } else {
                result = HttpClient.httpPost("http://192.168.6.51:8009/rgv2/sendTask", taskJson);
            }

            JSONObject jsonObject = JSON.parseObject(result);
            return Integer.parseInt(String.valueOf(jsonObject.get("msg")));

        } catch (IOException e) {
            e.printStackTrace();
            logger.error(new Date() + "WCS接口超时未响应！");
            return -1;
        }
    }

    /**
     * 发送搬运任务到RGV
     *
     * @param vehicleId     RGV
     * @param currTaskId    任务id
     * @param startPoint    起点
     * @param endPoint      终点
     * @param processNumber 工艺好
     * @return status状态码
     */
    private Integer sendTaskToRgv(Long vehicleId, Long currTaskId, Long startPoint, Long endPoint, Integer processNumber) {
        Map<String, Object> json = new HashMap();
        // taskType 1 搬运
        json.put("taskType", 1);
        json.put("sendRow", endPoint);
        json.put("pickRow", startPoint);
        json.put("taskNo", currTaskId);
        json.put("ideNumber", currTaskId);
        json.put("processNumber", processNumber);
        String taskJson = JSON.toJSONString(json);
        String result = null;
        logger.info("===执行任务：" + currTaskId + " ，即将执行车辆搬运任务，车辆：" + vehicleId + " ，起点：" + startPoint + " ,终点：" + endPoint + "===");
        try {
            if (vehicleId == 1) {
                result = HttpClient.httpPost("http://192.168.6.51:8009/rgv1/sendTask", taskJson);
            }
            if (vehicleId == 2) {
                result = HttpClient.httpPost("http://192.168.6.51:8009/rgv2/sendTask", taskJson);
            }
        } catch (IOException e) {
            e.printStackTrace();
            logger.error(new Date() + "WCS接口超时未响应！");
        }
        JSONObject jsonObject = JSON.parseObject(result);
        return Integer.parseInt(String.valueOf(jsonObject.get("msg")));
    }

    /**
     * 上报apms创建生产单并开始处理
     *
     * @param currTask 任务
     * @return 成功与否
     * @throws IOException
     */
    private Boolean apmsCreateProcess(CurrTask currTask) throws IOException {
        List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask.getId()));
        String code = kilnInfoServiceBiz.getById(currTask.getKilnId()).getCode();
        //通知apms创建一个生产单
        ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
        apmsCreateSheet.setWorkShopCode("BM");
        apmsCreateSheet.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsCreateSheet.setTechCode(currTask.getCraftCodeId().toString());
        //fixme 暂时没有正式user，使用测试账户
        apmsCreateSheet.setStartUser("QJJP03");
        ArrayList<CreateItem> createItems = new ArrayList<>();
        currTaskDetList.forEach(e -> {
            CreateItem createItem = new CreateItem();
            createItem.setItemCode(e.getIdenCardNum());
            createItem.setQuantity(e.getQuantity().intValue());
            createItem.setWeight(e.getWeight().doubleValue());
            createItems.add(createItem);
        });
        apmsCreateSheet.setStoveCode(code);
        apmsCreateSheet.setItems(createItems);
        logger.info(currTask.getId() + "开始创建apms生产单号");
        R<ApmsCreateProcessSheetVo> processSheet = apmsController.createProcessSheet(apmsCreateSheet);
        if (!processSheet.getData().getSuccess()) {
            logger.info(currTask.getId() + "创建失败，APMS报错：" + processSheet.getData().getMsg());
            webSocketServer.sendtoUser("创建失败，APMS报错：" + processSheet.getData().getMsg(), "1");
            return false;
        } else {
            logger.info(currTask.getId() + "创建apms生产单号成功");
            currTask.setSheetNo(processSheet.getData().getSheetNo());
            currTaskServiceBiz.updateById(currTask);
            //多步骤任务除了第一步写入sheetNo
            if (currTask.getIsAuto()==1){
                AutoExeTask autoExeTask = autoExeTaskServiceBiz.getById(currTask.getAutoExeTaskId());
                CurrTask secondCurrTask = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                secondCurrTask.setSheetNo(currTask.getSheetNo());
                currTaskServiceBiz.updateById(secondCurrTask);
                //三步任务
                if (autoExeTask.getProcessFlowType()==2){
                    CurrTask threeCurrTask = currTaskServiceBiz.getById(autoExeTask.getThTaskId());
                    threeCurrTask.setSheetNo(currTask.getSheetNo());
                    currTaskServiceBiz.updateById(threeCurrTask);
                }
            }
            return true;
        }
    }

    /**
     * 上报apms开始加工
     * @param currTask
     * @return
     */
    private Boolean apmsStartProcess(CurrTask currTask) {
        String code = kilnInfoServiceBiz.getById(currTask.getKilnId()).getCode();
        ApmsStartProcess apmsStartProcess=new ApmsStartProcess();
        apmsStartProcess.setStartUser("QJJP03");
        apmsStartProcess.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsStartProcess.setTechName(currTask.getCraftCodeId().toString());
        apmsStartProcess.setSheetNo(currTask.getSheetNo());
        apmsStartProcess.setStoveCode(code);
        R<ApmsStartProcess> apmsStartProcessR = apmsController.startProcess(apmsStartProcess);
        if (apmsStartProcessR.getCode() != 0) {
            logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单开始生产上报apms失败");
            return false;
        }
        logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单开始生产上报apms成功");
        return true;
    }

    /**
     * 上报apms-end
     * @param currTask
     * @return
     */
    private Boolean apmsEndProcess(CurrTask currTask){
        ApmsEndProcess apmsEndProcess = new ApmsEndProcess();
        apmsEndProcess.setEndTime(currTask.getEndTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsEndProcess.setEndUser("QJJP03");
        apmsEndProcess.setSheetNo(currTask.getSheetNo());
        R<ApmsEndProcessVo> apmsEndProcessVoR = apmsController.endProcess(apmsEndProcess);
        if (apmsEndProcessVoR.getCode() != 0) {
            logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单完成处理上报apms失败");
            return false;
        }
        logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单完成处理上报apms成功");
        return true;
    }

    /**
     * 上报apms-finish
     * @param currTask
     * @return
     */
    private Boolean apmsFinishProcess(CurrTask currTask){
        logger.info("任务：" + currTask.getTaskCode() + "开始上报apms完成生产单！");
        ApmsFinishProcessSheet apmsFinishProcessSheet = new ApmsFinishProcessSheet();
        apmsFinishProcessSheet.setSheetNo(currTask.getSheetNo());
        apmsFinishProcessSheet.setEndTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsFinishProcessSheet.setEndUser("QJJP03");
        List<FinishItem> finishItems = new ArrayList<>();
        List<CurrTaskDet> currTaskDets = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask.getId()));
        for (CurrTaskDet currTaskDet : currTaskDets
        ) {
            FinishItem finishItem = new FinishItem();
            BigDecimal bigDecimal = new BigDecimal(currTaskDet.getWeight());
            finishItem.setFinishWeight(bigDecimal.doubleValue());
            finishItem.setFinishQuantity(Math.round(currTaskDet.getQuantity()));
            finishItem.setItemCode(currTaskDet.getIdenCardNum());
            finishItems.add(finishItem);
        }
        apmsFinishProcessSheet.setItems(finishItems);
        R<ApmsFinishProcessSheetVo> apmsFinishProcessSheetVoR = apmsController.finishProcessSheet(apmsFinishProcessSheet);
        if (apmsFinishProcessSheetVoR.getCode() != 0) {
            logger.info("任务：" + currTask.getTaskCode() + "完成后上报apms完成生产单失败！");
            return false;
        }
        return true;
    }
    /**
     * 车辆是否在线
     * @param vehicleId 车辆id
     * @return true/false
     * @throws IOException  plc访问失败
     */
    private Boolean rgvOnline(Long vehicleId) throws IOException {
        String identifier=null;
        if (vehicleId==1){
            identifier="S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV1ToWCS.Online";
        }
        if (vehicleId==2){
            identifier="S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV2ToWCS.Online";
        }
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        json.put("identifier",identifier);
        String result = null;
        try {
            result = HttpClient.httpPost("http://192.168.6.51:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("获取车辆"+vehicleId+"当前是否在线失败");
        }
        Boolean online = Boolean.valueOf(JSONObject.parseObject(result).get("result").toString());
        return online;
    }

    /**
     * 传入车辆ID,检查另一辆车是否在线，如果在线，检查是否位置冲突，冲突则避让。若另一辆车不在线，则不检查是否冲突。
     * @param vehicleId
     * @param currTask
     * @throws IOException
     */
    private void rgvStatus(Long vehicleId,CurrTask currTask,String startPosition,String targetPosition) throws IOException {
        Long currTaskId = currTask.getId();
        if (vehicleId==1){
            Long anotherVehicleId=2L;
            Boolean rgv2Online = rgvOnline(anotherVehicleId);
            if (rgv2Online){
                //获取RGV2当前位置
                Double rgv2CurrPosition = RGVCurrPosition(2);
                //两辆小车是否冲突
                Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, startPosition, targetPosition, rgv2CurrPosition);
                if (conflictBoolean){
                    Integer status = moveRgv(2, currTaskId,  2,0);
                    logger.info(status.toString());
                    if (status!=1){
                        logger.info("===执行任务："+currTask.getTaskCode()+" 时车辆位置冲突，执行车辆位移任务，车辆"+anotherVehicleId+"移动到点位2，执行失败===");
                        webSocketServer.sendtoAll(LocalDateTime.now()+"执行任务："+currTask.getTaskCode()+" 时车辆位置冲突，执行车辆位移任务，车辆"+anotherVehicleId+"移动到点位2，执行失败");
                    }else {
                        logger.info("===执行任务："+currTaskId+" 时车辆位置冲突，执行车辆位移任务，车辆"+anotherVehicleId+"移动到点位2，执行成功===");
                    }
                }
            }else {
                logger.info("RGV2不在线，跳过检测是否避让。");
            }

        }else {
            Long anotherVehicleId=1L;
            Boolean rgv1Online = rgvOnline(anotherVehicleId);

            if (rgv1Online){

                //获取RGV1当前位置
                Double rgv1CurrPosition = RGVCurrPosition(1);
                //两辆小车是否冲突
                Boolean conflictBoolean = taskDistanceUtils.conflictForVehicle(vehicleId, startPosition, targetPosition, rgv1CurrPosition);
                if (conflictBoolean){
                    Integer status = moveRgv(1, currTaskId,  12,0);
                    logger.info(status.toString());
                    if (status!=1){
                        logger.info("===执行任务："+currTaskId+" 时车辆位置冲突，执行车辆位移任务，车辆"+anotherVehicleId+"移动到点位12，执行失败===");
                        webSocketServer.sendtoAll(LocalDateTime.now()+"执行任务："+currTask.getTaskCode()+" 时车辆位置冲突，执行车辆位移任务，车辆"+anotherVehicleId+"移动到点位12，执行失败");
                    }else {
                        logger.info("===执行任务："+currTaskId+" 时车辆位置冲突，执行车辆位移任务，车辆"+anotherVehicleId+"移动到点位12，执行成功===");
                    }
                }
            }else {
                logger.info("RGV1不在线，跳过检测是否避让。");
            }
        }
    }

    /**
     * 判断任务是否为单步骤任务，多步骤任务第一步
     * @param currTask 任务
     * @return 单步骤任务true，多步骤任务第一步true,其他false
     */
    public Boolean isFirstTask(CurrTask currTask){
        if (currTask.getIsAuto()==1){
            int count = autoExeTaskServiceBiz.count(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.F_TASK_ID, currTask.getId()));
            return count>0;
        }else {
            return true;
        }
    }


}
