/*
 * Copyright (c) 2019.
 * http://www.ulabcare.com
 */

package com.mt.wms.passport.vo;

import com.mt.wms.core.vo.CommonVo;
import com.mt.wms.core.vo.PageVo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.time.LocalDateTime;

/**
 * <p>
 * 登录会话
 * </p>
 *
 * @author jiff
 * @since 2019-07-20
 */
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@Data
@Builder
@ApiModel("会话视图对象")
public class LoginSessionVo extends CommonVo implements PageVo.ConvertVo {

    private static final long serialVersionUID = 1L;

    /**
     * 主键，自增
     */
    @ApiModelProperty(value = "id")
    private Long id;

    /**
     * 会话ID
     */
    @ApiModelProperty("会话ID")
    private String sessionId;

    /**
     * 父会话ID，漫游登录时需要保存发起漫游用户的会话ID
     */
    @ApiModelProperty("父会话ID，漫游登录时需要保存发起漫游用户的会话ID")
    private String parentSessionId;

    /**
     * 父用户名称
     */
    @ApiModelProperty("父用户名称")
    private String parentUserName;

    /**
     * 统一用户ID
     */
    @ApiModelProperty("统一用户ID")
    private Long passportUserId;

    /**
     * 用户ID
     */
    @ApiModelProperty("用户ID")
    private Long userId;

    /**
     * 合作伙伴ID，合作伙伴用户或者医院用户登录时会存储
     */
    @ApiModelProperty("合作伙伴ID，合作伙伴用户或者医院用户登录时会存储")
    private Long partnerId;

    /**
     * 医院ID
     */
    @ApiModelProperty("医院ID")
    private Long hospitalId;

    /**
     * 组织ID
     */
    @ApiModelProperty("组织ID")
    private Long orgId;

    /**
     * 手机号
     */
    @ApiModelProperty("手机号")
    private String mobile;

    /**
     * 用户名称
     */
    @ApiModelProperty("用户名称")
    private String userName;

    /**
     * 合作伙伴名称
     */
    @ApiModelProperty("合作伙伴名称")
    private String partnerName;

    /**
     * 医院名称
     */
    @ApiModelProperty("医院名称")
    private String hospitalName;

    /**
     * 组织名称
     */
    @ApiModelProperty("组织名称")
    private String orgName;

    /**
     * 用户类型：1、平台用户，2、合作伙伴用户，3、医院用户，4、患者
     */
    @ApiModelProperty("用户类型：1、平台用户，2、合作伙伴用户，3、医院用户，4、患者")
    private Integer userType;

    /**
     * 应用类型：1、pc，2、app，3、wechat，4、miniapp
     */
    @ApiModelProperty("应用类型：1、pc，2、app，3、wechat，4、miniapp")
    private Integer appType;

    /**
     * 应用编码：暂未使用
     */
    @ApiModelProperty("应用编码：暂未使用")
    private Integer appCode;

    /**
     * 登录类型：1、自主登录，2、漫游登录
     */
    @ApiModelProperty("登录类型：1、自主登录，2、漫游登录")
    private Integer loginType;

    /**
     * 登录时间
     */
    @ApiModelProperty("登录时间")
    private LocalDateTime loginTime;

    /**
     * 设备类型
     */
    @ApiModelProperty("设备类型")
    private String deviceClass;

    /**
     * 设备名称
     */
    @ApiModelProperty("设备名称")
    private String deviceName;

    /**
     * 设备品牌
     */
    @ApiModelProperty("设备品牌")
    private String deviceBrand;

    /**
     * 操作系统
     */
    @ApiModelProperty("操作系统")
    private String os;

    /**
     * 浏览器
     */
    @ApiModelProperty("浏览器")
    private String browser;

    /**
     * 国家
     */
    @ApiModelProperty("国家")
    private String country;

    /**
     * 省份
     */
    @ApiModelProperty("省份")
    private String province;

    /**
     * 地市
     */
    @ApiModelProperty("地市")
    private String city;

    /**
     * 区县
     */
    @ApiModelProperty("区县")
    private String county;

    /**
     * 地址
     */
    @ApiModelProperty("地址")
    private String address;

    /**
     * 精度
     */
    @ApiModelProperty("精度")
    private String lng;

    /**
     * 纬度
     */
    @ApiModelProperty("纬度")
    private String lat;

    /**
     * 网络服务提供商
     */
    @ApiModelProperty("网络服务提供商")
    private String isp;

    /**
     * IP地址
     */
    @ApiModelProperty("IP地址")
    private String ip;

    /**
     * 用户代理，存放用户登录时客户端信息
     */
    @ApiModelProperty("用户代理")
    private String userAgent;

}
