/*
 * Copyright (c) 2018.
 * http://www.ulabcare.com
 */

package com.mt.wms.upms.params;

import com.mt.wms.core.enums.HttpMethodEnum;
import com.mt.wms.core.enums.ResourceCategoryEnum;
import com.mt.wms.core.enums.SubsystemEnum;
import com.mt.wms.core.base.BaseParam;
import com.mt.wms.core.validator.constraints.Enum;
import com.mt.wms.core.validator.groups.UpdateGroup;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

/**
 * <p>
 * 资源表
 * </p>
 *
 * @author jiff
 * @date 2018/11/22
 * @since 1.0
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value = "资源表", description = "用于新增和更新资源信息")
public class ResourceParam extends BaseParam {

    private static final long serialVersionUID = 1L;

    /**
     * 主键，自增
     */
    @ApiModelProperty(value = "主键，更新时需要填写", example = "1")
    @NotNull(message = "资源ID不能为空", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 服务ID
     */
    @ApiModelProperty(value = "服务ID", required = true, example = "1")
    @NotNull(message = "服务ID不能为空")
    private Long serviceId;

    /**
     * 名称
     */
    @ApiModelProperty(value = "名称", required = true, example = "新增资源")
    @NotBlank(message = "名称不能为空")
    private String name;

    /**
     * 请求地址，可以用于前端ajax调用，后端用于做权限控制
     */
    @ApiModelProperty(value = "请求地址，可以用于前端ajax调用，后端用于做权限控制", required = true, example = "/platform-upms/resource/add")
    @NotBlank(message = "请求地址不能为空")
    private String url;

    /**
     * 请求类型：GET, HEAD, POST, PUT, PATCH, DELETE, OPTIONS, TRACE
     */
    @ApiModelProperty(value = "请求类型：GET, HEAD, POST, PUT, PATCH, DELETE, OPTIONS, TRACE", required = true, example = "post")
    @NotBlank(message = "请求方法不能为空")
    @Enum(message = "非法的请求类型", target = {HttpMethodEnum.class})
    private String method;

    /**
     * 子系统:0、common，1、platform，2、partner，3、 hospital，4、patient
     */
    @ApiModelProperty(value = "子系统:0、common，1、platform，2、partner，3、 hospital，4、patient", required = true, example = "0")
    @NotNull(message = "子系统类型不能为空")
    @Enum(message = "非法的子系统类型", target = {SubsystemEnum.class})
    private Integer subsystem;

    /**
     * 分类:0、common，1、pc，2、app，3、wechat，4、miniapp
     */
    @ApiModelProperty(value = "分类:0、common，1、pc，2、app，3、wechat，4、miniapp", required = true, example = "0")
    @NotNull(message = "资源类型不能为空")
    @Enum(message = "非法的资源分类", target = {ResourceCategoryEnum.class})
    private Integer category;

    /**
     * 备注
     */
    @ApiModelProperty("备注")
    private String remark;

}
